# -*- coding: utf-8 -*-
import re
import os

import autocomplete_light
from django.conf import settings
from django import forms
from django.utils.translation import ugettext_lazy as _
from jmb.core.middleware.thread_local import get_request
from jmb.jadmin import JumboActionForm
from . import models

class DocumentForm(forms.ModelForm):
    projects = forms.ModelMultipleChoiceField(
        queryset=models.DocumentProject.objects.all(), required=False,
        widget=autocomplete_light.MultipleChoiceWidget('DocumentProjectAutocomplete')
    )

    class Meta:
        model = models.Document
        fields = ['file', 'category', 'title', 'tipology', 'projects', 'status']

    def __init__(self, *args, **kwargs):
        super(DocumentForm, self).__init__(*args, **kwargs)
        self.fields['title'].required = False

    @property
    def filing_fields(self):
        fields = [f for f in self if f.name in ('category', 'tipology', 'projects', 'status')]
        return fields

    def save(self, production_pk=None, *args, **kw):
        ## An image is connected or to an artist or to a production
        obj = super(DocumentForm, self).save(*args, **kw)
        obj.organization_id = settings.DOCUMENT_ORGANIZATION_DEFAULT
        obj.save()
        return obj

    def clean_title(self):
        name = re.sub('[_-]', ' ', self.cleaned_data['file'].name)
        name, ext = os.path.splitext(name)
        return name

class DocumentBulkForm(forms.ModelForm):

    def get_info_from_request(self, field):
        """Get info from the request sent from ajax by fileupload
        """
        request = get_request()
        filename = self.cleaned_data.get("file").name
        originallist = request.POST.getlist("original")
        fieldlist = request.POST.getlist(field)
        indice = originallist.index(filename)
        return fieldlist[indice]

    def clean_title(self):
        return self.get_info_from_request("title")

    def clean_category(self):
        pk = self.get_info_from_request("category")
        return models.Category.objects.get(pk=pk)


class DocumentActionForm(JumboActionForm):
    projects = forms.ModelMultipleChoiceField(
        queryset=models.DocumentProject.objects.all(), required=False,
        widget=autocomplete_light.MultipleChoiceWidget('DocumentProjectAutocomplete')
    )
    category = forms.ModelChoiceField(
        queryset=models.Category.objects.all(),
        required=False,
    )
    tipology = forms.ModelChoiceField(
        queryset=models.Tipology.objects.all(),
        required=False,
    )
    organization = forms.ModelChoiceField(
        initial=settings.DOCUMENT_ORGANIZATION_DEFAULT,
        queryset=models.Organization.objects.all(),
        required=False
    )
    direction = forms.ChoiceField(
        choices=settings.DOCUMENT_STATUS_DOC,
        required=False
    )
    status = forms.ChoiceField(
        choices=settings.DOCUMENT_SUPPORTO_DOC,
        required=False
    )
    keywords = forms.CharField(max_length=200, required=False)
    fields_map = {
        'export_action': ['output_type'],
        'join': [],
        'split': [],
        'filing': ['category', 'tipology', 'organization', 'projects',
                   'direction', 'status', 'keywords'],
    }

    class Meta:
        model = models.Document
        exclude = []
